package
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.filters.BlurFilter;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.ui.Keyboard;
	import nape.geom.Vec2;
	import nape.phys.Body;
	import nape.phys.BodyList;
	import nape.phys.BodyType;
	import nape.phys.Material;
	import nape.shape.Circle;
	import nape.shape.Polygon;
	import nape.space.Space;
	import nape.util.BitmapDebug;
	
	/**
	 * ...
	 * @author Faisal
	 */
	
	[SWF(frameRate="30",width=480,height=640,backgroundColor="0xCCCCCC")]
	
	public class Main extends Sprite
	{
		private var space:Space;
		private var debug:BitmapDebug;
		private var stepTime:Number = 1 / 30;
		
		private var circleBodyList:BodyList;
		
		private var drawCanvasData:BitmapData;
		private var drawCanvasDataCopy:BitmapData;
		
		private var drawCanvas:Bitmap;
		private var drawCanvas2:Bitmap;
		
		private var circleSprite:Sprite;
		
		private var transMatrix:Matrix;
		
		private var waterRadius:Number = 6;
		private var numOfParticles:int = 200;
		
		private var blurFilter:BlurFilter = new BlurFilter(15, 15, 3);
		
		private var point:Point = new Point(0, 0);
		
		private var mouseXY:Vec2 = new Vec2();
		
		private var milkText:TextField;
		
		public function Main():void
		{
			if (stage)
				init();
			else
				addEventListener(Event.ADDED_TO_STAGE, init);
		}
		
		private function init(e:Event = null):void
		{
			removeEventListener(Event.ADDED_TO_STAGE, init);
			// entry point
			
			var gravity:Vec2 = new Vec2(0, 500);
			space = new Space(gravity);
			//2.create debugshape			
			debug = new BitmapDebug(480, 640, 0x0000ff, true);
			
			//this.addChild(debug.display);
			
			drawCanvasData = new BitmapData(480, 640, false, 0xFFFFFFFF);
			
			drawCanvasDataCopy = new BitmapData(480, 640, true, 0xFFFFFFFF);
			
			drawCanvas = new Bitmap(drawCanvasData);
			
			drawCanvas2 = new Bitmap(drawCanvasDataCopy);
			
			milkText = new TextField();
			var textFormat:TextFormat = new TextFormat("verdana", 80, 0xFFFFFF, true);
			milkText.defaultTextFormat = textFormat;
			milkText.text = "MILK";
			milkText.width = 350;
			milkText.selectable = false;
			milkText.x = 110;
			milkText.y = 250;
			
			addChild(milkText);
			
			this.addChild(drawCanvas2);
			
			//this.addChild(drawCanvas);
			
			circleSprite = new Sprite();
			circleSprite.graphics.beginFill(0xFF0000);
			circleSprite.graphics.drawCircle(0, 0, waterRadius);
			circleSprite.graphics.endFill();
			
			transMatrix = new Matrix();
			transMatrix.tx = 100;
			transMatrix.ty = 100;
			drawCanvasData.draw(circleSprite, transMatrix);
			
			circleBodyList = new BodyList();
			for (var i:int = 0; i < numOfParticles; i++)
			{
				var circleBody:Body = new Body(BodyType.DYNAMIC, new Vec2(waterRadius+Math.random()*460, waterRadius+Math.random()*620));
				var circShape:Circle = new Circle(waterRadius);
				circleBody.shapes.push(circShape);
				circleBody.space = space;
				circleBodyList.add(circleBody);
			}
			//trace(circleBodyList);
			createWrapWall();
			
			addEventListener(Event.ENTER_FRAME, loop);
			stage.addEventListener(KeyboardEvent.KEY_UP, onKeyUp);
			addChild(new Stats());
		}
		
		private function onKeyUp(e:KeyboardEvent):void 
		{
			if (e.keyCode == Keyboard.LEFT)
			{
				space.gravity = Vec2.weak(-500, 0);
			}
			else if (e.keyCode == Keyboard.RIGHT)
			{
				space.gravity = Vec2.weak(500, 0);
			}
			else if (e.keyCode == Keyboard.UP)
			{
				space.gravity = Vec2.weak(0, -500);
			}
			else if (e.keyCode == Keyboard.DOWN)
			{
				space.gravity = Vec2.weak(0, 500);
			}
		}
		
		private function createWrapWall():void
		{
			createBox(stage.stageWidth / 2, 0, stage.stageWidth, 10, BodyType.STATIC);
			createBox(stage.stageWidth / 2, stage.stageHeight, stage.stageWidth, 10, BodyType.STATIC);
			createBox(0, stage.stageHeight / 2, 10, stage.stageHeight, BodyType.STATIC);
			createBox(stage.stageWidth, stage.stageHeight / 2, 10, stage.stageHeight, BodyType.STATIC);
		}
		
		private function loop(e:Event):void
		{
			mouseXY.x = stage.mouseX;
			mouseXY.y = stage.mouseY;
			
			space.step(stepTime);
			drawCanvasData.fillRect(drawCanvasData.rect,0x00000000);
			for (var i:int = 0; i < numOfParticles; i++)
			{
				transMatrix.tx = circleBodyList.at(i).position.x;
				transMatrix.ty = circleBodyList.at(i).position.y;
				drawCanvasData.draw(circleSprite, transMatrix);
				
				var body:Body = circleBodyList.at(i);
				var dist:Vec2 = body.position.sub(mouseXY);
				if(dist.length<40)
				{
					body.applyImpulse(dist);
				}
			}
			drawCanvasData.applyFilter(drawCanvasData, drawCanvasData.rect, new Point(0, 0), blurFilter);
			
			drawCanvasDataCopy.fillRect(drawCanvasData.rect,0x00000000);
			drawCanvasDataCopy.threshold(drawCanvasData, drawCanvasData.rect, point, ">", 0XFF2b2b2b, 0x55FFFFFF, 0xFFFFFFFF, false);
			drawCanvasDataCopy.threshold(drawCanvasData, drawCanvasData.rect, point, ">", 0XFF2c2c2c, 0xBBFFFFFF, 0xFFFFFFFF, false);
			drawCanvasDataCopy.threshold(drawCanvasData, drawCanvasData.rect, point, ">", 0XFF2d2d2d, 0xFFFFFFFF, 0xFFFFFFFF, false);
			debug.clear();
			debug.draw(space);
			debug.flush();
		}
		
		private function createBox(posX:Number, posY:Number, width:Number, height:Number, type:BodyType):Body
		{
			var box:Body = new Body(type, new Vec2(posX, posY));
			var shape:Polygon = new Polygon(Polygon.box(width, height));
			box.shapes.push(shape);
			box.space = space;
			return box;
		}
	}
}